//+------------------------------------------------------------------+
//|                                                 MultiVolumes.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, TheXpert"
#property link      "theforexpert@gmail.com"

#property indicator_separate_window
#property indicator_buffers 7

#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_color3 Blue
#property indicator_color4 Orange
#property indicator_color5 White
#property indicator_color6 DarkGray
#property indicator_color7 Purple

extern int ATRPeriod = 9;
extern int MaxCount = 1000;

#include <SolverExternal.mqh>

string Currencies[] = {"EUR", "USD", "JPY", "GBP", "CAD", "CHF", "AUD"};

double EUR[];
double USD[];
double JPY[];
double GBP[];
double CAD[];
double CHF[];
double AUD[];

datetime LastTime;

void SetThickLine(string currency)
{
   int size = ArraySize(Currencies);
   
   for (int i = 0; i < size; i++)
   {
      if (currency == Currencies[i])
      {
         SetIndexStyle(i, DRAW_LINE, STYLE_SOLID, 3);
      }
   }
}

void MakePatterns(int ID, int index)
{
   int size = ArraySize(Currencies);

   for (int i = 0; i < size; i++)
   {
      for (int j = 0; j < size; j++)
      {
         string pair = Currencies[i] + Currencies[j];
         
         if (MarketInfo(pair, MODE_ASK) != 0)
         {
            double pattern[];
            ArrayResize(pattern, size);
            ArrayInitialize(pattern, 0);
            pattern[i] = 1;
            pattern[j] = 1;
            double value[1];
            value[0] = iCustom(pair, Period(), "NATR", ATRPeriod, 0, iBarShift(pair, Period(), Time[index]));
            
            if (value[0] != EMPTY_VALUE)
            {
               AddPatternsToSolver(ID, 1, size, pattern, 1, value);
            }
         }
      }
   }
}

int init()
{
   LastTime = 0;
   
   SetIndexBuffer(0, EUR);
   SetIndexBuffer(1, USD);
   SetIndexBuffer(2, JPY);
   SetIndexBuffer(3, GBP);
   SetIndexBuffer(4, CAD);
   SetIndexBuffer(5, CHF);
   SetIndexBuffer(6, AUD);
   
   int size = ArraySize(Currencies);
   for (int i = 0; i < size; i++)
   {
      SetIndexLabel(i, Currencies[i]);
      SetIndexStyle(i, DRAW_LINE, STYLE_DASH, 1);
   }
   
   SetThickLine(StringSubstr(Symbol(), 0, 3));
   SetThickLine(StringSubstr(Symbol(), 3, 3));

   return(0);
}

int start()
{
   if (LastTime == Time[0]) return (0);
   LastTime = Time[0];
   
   int size = ArraySize(Currencies);
   
   int ID = GetTickCount();
   while(!RegisterUsage(ID, size))
   {
      ID++;
   }
   
   int toCount = Bars - IndicatorCounted();
   if (toCount > MaxCount) toCount = MaxCount;
   
   for (int i = toCount - 1; i > 0; i--)
   {
      ClearSolver(ID);
   
      MakePatterns(ID, i);

      double solution[];
      ArrayResize(solution, size);
      SolveProblem(ID, size, solution);
   
      for (int j = 0; j < size; j++)
      {
         if (solution[j] == 0) solution[j] = EMPTY_VALUE;
      }
   
      EUR[i] = solution[0];
      USD[i] = solution[1];
      JPY[i] = solution[2];
      GBP[i] = solution[3];
      CAD[i] = solution[4];
      CHF[i] = solution[5];
      AUD[i] = solution[6];
   }
   
   UnregisterUsage(ID);
   
   return(0);
}